messagetemplates

package
v0.24.0 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Mar 11, 2026 License: MPL-2.0 Imports: 7 Imported by: 0

Documentation

Overview

Package messagetemplates TODO

Index

Constants

const (
	LengthTemplate templateKey = iota + 1
	MinLengthTemplate
	MaxLengthTemplate
	EQTemplate
	NEQTemplate
	GTTemplate
	GTETemplate
	LTTemplate
	LTETemplate
	EqualPropertiesTemplate
	GTPropertiesTemplate
	GTEPropertiesTemplate
	LTPropertiesTemplate
	LTEPropertiesTemplate
	GTComparablePropertiesTemplate
	GTEComparablePropertiesTemplate
	LTComparablePropertiesTemplate
	LTEComparablePropertiesTemplate
	DurationPrecisionTemplate
	ForbiddenTemplate
	OneOfTemplate
	NotOneOfTemplate
	OneOfPropertiesTemplate
	MutuallyExclusiveTemplate
	MutuallyDependentTemplate
	RequiredTemplate
	StringNonEmptyTemplate
	StringMatchRegexpTemplate
	StringDenyRegexpTemplate
	StringEmailTemplate
	StringMACTemplate
	StringIPTemplate
	StringIPv4Template
	StringIPv6Template
	StringCIDRTemplate
	StringCIDRv4Template
	StringCIDRv6Template
	StringJSONTemplate
	StringContainsTemplate
	StringExcludesTemplate
	StringStartsWithTemplate
	StringEndsWithTemplate
	StringTitleTemplate
	StringGitRefTemplate
	StringFileSystemPathTemplate
	StringFilePathTemplate
	StringDirPathTemplate
	StringMatchFileSystemPathTemplate
	StringRegexpTemplate
	StringCrontabTemplate
	StringDateTimeTemplate
	StringTimeZoneTemplate
	StringKubernetesQualifiedNameTemplate
	URLTemplate
	SliceUniqueTemplate
	UniquePropertiesTemplate
)

Variables

This section is empty.

Functions

func AddFunctions

func AddFunctions(tpl *template.Template) *template.Template

AddFunctions adds a set of custom functions to the provided template. These functions are used by builtin templates.

func Get

func Get(key templateKey) *template.Template

Get returns a message template by its key. If the template is not found, it panics. The first time a template is requested, it is parsed and stored in the cache. Custom functions and dependencies are added to the template automatically.

Types

This section is empty.

Jump to